/******************************************************************************
*
* DEFCON 17 BADGE, BLIMP EDITION
*
* Filename:		  DC17_Badge.c
* Author:		  Room 863, Riviera Hotel
* Revision:		  1.1
* Last Updated:   August 2, 2009
*
* Description:	  Main File for the DEFCON 17 Badge (Freescale MC56F8006)
* Notes:          "Slightly" modified version of the Kingpin release.
*
* See DC17_Badge.h for more inf0z...		
*******************************************************************************/

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
//#include "PWMRed.h"
//#include "PWMGreen.h"
//#include "PWMBlue.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "DC17_Badge.h"


/****************************************************************************
 ************************** Global variables ********************************
 ***************************************************************************/
 
bool isMaster = false;
int master_led_ctr;

// Badge 
badge_state_type badge_state = I2C; // begin in I2C (multi-badge) mode on power-up
badge_type_struct badge_type;
struct packed_flags flags;

// Microphone
unsigned int freqcount = 0;
word	micval = 0;
word  micval_offset;

// FFT
dfr16_tRFFTStruct RFFT;
dfr16_tRFFTStruct *pRFFT = &RFFT;
CFrac16 FFTInplaceBuf[MAX_RFFT_LEN/2];
unsigned long FFTVal[3];

// A/D
unsigned int AdcBuffer[ADC_MAX_BUFFER_SIZE];
unsigned int AdcWriteIndex = 0;
unsigned int AdcReadIndex = 0;

// LED
//led_state_type LEDstate = RED;
//unsigned int LEDptr = 0;

// RNG
//unsigned int randtime = 0;
//int randval; 

// Sleep
byte threshold_failure;
word threshold_count;

// Modes
//unsigned int soscount = 0;
//unsigned int dancetmr = 0; 

// Data sanitation
//int calibration_cycles = 1000;
//int locut[3];

// Running average
unsigned long FFT_bins_averages[3];
byte FFT_bin_result;

// LED Fun
bool led_anim_active;
int led_frame;

// Motor ctl
int motor_delay_ctr;
bool motor_on;
bool okay_to_listen;

/****************************************************************************
 ************************** Functions ***************************************
 ***************************************************************************/

void dc17_badge(void)
{ 
	dc17_init();
  
	// check to see if badges are connected to each other and, if so, enter badge-to-badge mode (master)
  // this happens only once on power-up
  // every slave badge that is connected to the bus *must* be powered on or the bus will fail like a whale
	//if (badge_type == HUMAN) dc17_i2c_check(); 
  
  // begin normal operation
  badge_state = FFT; // enable FFT-related functions in the interrupts
  
  /* patrick : keeps in party mode */
  
  //flags.sleep = 0;		 // stay awake
  //TI1_Enable();				 // make sure TMR1 is only enabled when the device is completely awake (to avoid flickering LED while sleeping)
  //dancetmr = DANCE_TMR;	// set our timer  
  
  /* end */
  
  //PORTB1OUT_ClrVal();
    
  while(1)
  {  	
  	//if (badge_type != HUMAN) if (flags.i2c_rx) dc17_i2c_slave(); // if i2c data packet received, enter badge-to-badge mode (slave)
  
  
  	// stop motor
  	// delay (1800ms)
  	// do fft
  	// decide motor dir
  
  
  
    if (okay_to_listen)
    {
    	dc17_fft();
    	dc17_process_fft();
    }
   	decide_motor_direction();
  

    //decide_motor_direction();
    //if master, consider panicking!
    

  }
}


/**************************************************************/

void dc17_fft(void) // do tha' FFT
{
	unsigned int  i, j;
 	Result 				res;
 	
	unsigned long   Temp1, Temp2;    

	//variables for calculating the bin averages
	static unsigned long FFT_bin_running_sums[3];
	static unsigned int FFTs_summed;
	#define FFT_SAMPLES_PER_AVERAGE 32

 	if (flags.PerformFFT == true)
	{      
		//BEGIN JOE'S MADNESS
   		// perform the FFT (N = 8)
    	res = dfr16RFFT (pRFFT, (Frac16 *)&AdcBuffer[AdcReadIndex], (dfr16_sInplaceCRFFT *)&FFTInplaceBuf[0]);

    	for (i = 1; i <= ((MAX_RFFT_LEN/2) - 1); i++) // ignore bin 0
    	{
      		// calculate the power of the ith element (bin) of the FFT
      		// M = sqrt(real^2 + imag^2)
    			// P = M^2
 	   			// since the power of a frequency element of the FFT is the square of the magnitude, we can just skip
		  		// the square root while calculating the magnitude and we're left with the power. 
					// we ignore the phase component of the FFT.

      		Temp1 = abs_s(FFTInplaceBuf[i].real);
      		Temp1 = Temp1*Temp1;
        
		      Temp2 = abs_s(FFTInplaceBuf[i].imag);
		      FFTVal[i-1] = (Temp2*Temp2) + Temp1;

		}	
		//END THE MADNESS

		//increment running sum
		for (j = 0; j < 3; j++)
		{
			FFT_bin_running_sums[j] += FFTVal[j];
		}
		FFTs_summed++;
	
		//update the average, if our sum is big enough
		if (FFTs_summed >= FFT_SAMPLES_PER_AVERAGE)
		{
			FFTs_summed = 0;
			
			for (j = 0; j < 3; j++)
			{
				FFT_bins_averages[j] = FFT_bin_running_sums[j] / FFT_SAMPLES_PER_AVERAGE;
				FFT_bin_running_sums[j] = 0;
			}
		}		
		


		/*Term1_SendStr("\n\r0: ");
		Term1_SendNum((long)FFTVal[0]);
		Term1_SendStr("\t1: ");
		Term1_SendNum((long)FFTVal[1]);
		Term1_SendStr("\t2: ");
		Term1_SendNum((long)FFTVal[2]);*/		
		/*if (FFTAvgCtr % 10 == 0)
		for (j = 0; j < 3; j++)
		{
			Term1_SendStr("DEBUG: ");
			
			Term1_SendNum((int32_t) FFT_bins_averages[0]);
			Term1_SendStr(" ");
			Term1_SendNum((int32_t) FFT_bins_averages[1]);
			Term1_SendStr(" ");
			Term1_SendNum((int32_t) FFT_bins_averages[2]);
			Term1_SendStr("\r\n");
		}*/
		
				
		//ADC voodoo
	    AdcReadIndex += MAX_RFFT_LEN;

	    if (AdcReadIndex >= ADC_MAX_BUFFER_SIZE)
	    {
	     	AdcReadIndex = 0;
	    }

	    flags.PerformFFT = false;
	}
}


void dc17_process_fft(void) // do stuff based on the calculated FFT data
{
	int i, sum;
	unsigned int avg;

	#define FFT_LO_CUT 800
	static unsigned int FFT_winners_index;
	#define FFT_WINNERS_PER_AVERAGE 10
	static byte FFT_running_winners[FFT_WINNERS_PER_AVERAGE];

	//decide a winner
	byte winner;
	if (FFT_bins_averages[0] > FFT_bins_averages[1] && FFT_bins_averages[0] > FFT_bins_averages[2]) // Bin 0 greatest
	{
		winner = 0;
	}
	else if (FFT_bins_averages[1] > FFT_bins_averages[2]) // Bin 1 greatest
	{
		winner = 1;
	}
	else // Bin 2 greatest
	{		
		winner = 2;
	}
	
	//if the winner is awesome, add it to running winners
	if (FFT_bins_averages[winner] > FFT_LO_CUT) 
	{
		FFT_running_winners[FFT_winners_index] = winner;
		FFT_winners_index= (FFT_winners_index+1)%FFT_WINNERS_PER_AVERAGE;
	}
	
	//average the winners
	sum = 0;
	for (i = 0; i < FFT_WINNERS_PER_AVERAGE; i++)
	{
		sum += (int)FFT_running_winners[i];	
		/*Term1_SendStr("\tsum @ ");
		Term1_SendNum(i);
		Term1_SendStr(": ");
		Term1_SendNum(sum);
		Term1_SendStr("\r\n");*/
	}
		
	avg = (unsigned int) (sum / FFT_WINNERS_PER_AVERAGE);
	
	//and that's our FINAL ANSWER
	FFT_bin_result = (byte) avg;
	
	
	//moar debug!!!1!!
	/*if (true)//(avg != FFT_bin_result)
	{
		Term1_SendNum(sum);
		Term1_SendStr(": ");
		Term1_SendNum(avg);
		Term1_SendStr(": ");
		
		Term1_SendNum((int32_t) FFT_bins_averages[0]);
		Term1_SendStr(" ");
		Term1_SendNum((int32_t) FFT_bins_averages[1]);
		Term1_SendStr(" ");
		Term1_SendNum((int32_t) FFT_bins_averages[2]);
		
		Term1_SendStr("\n\r");
	}	
	*/
}

void decide_motor_direction(void) // sleep, if required
{
	if(!motor_on)
	{
		MotorPin_ClrVal();
	}
	else
	{
		if (FFT_bins_averages[0] < 70000)//if(FFT_bin_result == 0)
		{
			MotorPin_ClrVal();
			Term1_SendNum((long)FFT_bins_averages[0]);
			Term1_SendStr("Motor OFF\r\n");
		}
		else
		{
			MotorPin_SetVal();
			Term1_SendNum((long)FFT_bins_averages[0]);
			Term1_SendStr(" Motor ON\r\n");
		}
	}
}

void do_led_anim(void)
{
	// Pins: 10, 11, 12
	
	//int sect = led_frame / 3;
	
	switch (led_frame)
	{
	case 0:
		LED1_SetVal();
		LED2_ClrVal();
		LED3_ClrVal();
		break;
	case 1:
		LED1_ClrVal();
		LED2_SetVal();
		LED3_ClrVal();
		break;
	case 2:
		LED1_ClrVal();
		LED2_ClrVal();
		LED3_SetVal();
		break;
	}
	
	if (++led_frame >= 3)
	{
		/*LED1_ClrVal();
		LED2_ClrVal();
		LED3_ClrVal();
		
		led_anim_active = false;*/
		led_frame = 0;
	}
}

void slave_interrupt(void)
{
	
}

/**************************************************************/

void dc17_i2c_check(void)
{
	/*
	Result 					res;
	word						i2cret; // return value from i2c function call	
  byte						tmp[6]; // temporary buffer for i2c tx/rx
  
  // send a test broadcast packet to see if we receive a response back from any badges
	flags.i2c_nack = 0;
     
	tmp[0] = 'D';
	tmp[1] = 'E'; 
	tmp[2] = 'F';
	tmp[3] = 'C'; 
	tmp[4] = 'O';
	tmp[5] = 'N';
	BADGEI2C_SelectSlave(0);	// set slave address to broadcast (0)	
	res = BADGEI2C_SendBlock(&tmp, 6, &i2cret);		  
	delay_ms(1);  
	if (!flags.i2c_nack) // if NACK flag not set, then other badges must be connected
	{
	  //Term1_SendStr("Multi-Badge Master.\n\r");
		
		
		
 		// let's SEND data...
		while(1) // ...forever!
		{
		  while (!flags.ptr_update){}; // wait until the RGB blending pointer has been updated...
			
			// do a neat LED animation to led people know we're in multi-badge mode
   		switch (LEDptr % 8)
  		{		
  			case 0:
				 	PWMRed_SetRatio16(OFF);
				 	PWMGreen_SetRatio16(OFF);
				 	PWMBlue_SetRatio16(OFF);
				 	tmp[0] = tmp[1] = tmp[2] = tmp[3] = tmp[4] = tmp[5] = 0xFF; // simple way to set the bytes to transfer
					break;
  			case 1:
				 	PWMRed_SetRatio16(OFF);
				 	PWMGreen_SetRatio16(OFF);
				 	PWMBlue_SetRatio16(ON);
				 	tmp[0] = tmp[1] = tmp[2] = tmp[3] = tmp[5] = 0xFF;
					tmp[4] = 0x4F;
				 	break;
  			case 2:
				 	PWMRed_SetRatio16(OFF);
				 	PWMGreen_SetRatio16(ON);
				 	PWMBlue_SetRatio16(ON);
				 	tmp[0] = tmp[1] = tmp[3] = tmp[5] = 0xFF;
					tmp[2] = tmp[4] = 0x4F;
				 	break;
  			case 3:
				 	PWMRed_SetRatio16(OFF);
				 	PWMGreen_SetRatio16(ON);
				 	PWMBlue_SetRatio16(OFF); 
				 	tmp[0] = tmp[1] = tmp[3] = tmp[4] = tmp[5] = 0xFF;
					tmp[2] = 0x4F;
  				break;
  			case 4:
				 	PWMRed_SetRatio16(ON);
				 	PWMGreen_SetRatio16(ON);
				 	PWMBlue_SetRatio16(OFF);  			
				 	tmp[1] = tmp[3] = tmp[4] = tmp[5] = 0xFF;
					tmp[0] = tmp[2] = 0x4F;
  				break;
  			case 5:
				 	PWMRed_SetRatio16(ON);
				 	PWMGreen_SetRatio16(OFF);
				 	PWMBlue_SetRatio16(OFF);  			
				 	tmp[1] = tmp[2] = tmp[3] = tmp[4] = tmp[5] = 0xFF;
					tmp[0] = 0x4F;
  				break;
  			case 6:
				 	PWMRed_SetRatio16(ON);
				 	PWMGreen_SetRatio16(OFF);
				 	PWMBlue_SetRatio16(ON);  			
				 	tmp[1] = tmp[2] = tmp[3] = tmp[5] = 0xFF;
					tmp[0] = tmp[4] = 0x4F;
  				break; 				
  			case 7:
				 	PWMRed_SetRatio16(ON);
				 	PWMGreen_SetRatio16(ON);
				 	PWMBlue_SetRatio16(ON);  			
				 	tmp[1] = tmp[3] = tmp[5] = 0xFF;
					tmp[0] = tmp[2] = tmp[4] = 0x4F;
  				break; 
 			}      
			
			//BADGEI2C_SelectSlave(UBER); // use this function to direct all subsequent calls to a particular slave device (instead of broadcasting)
		  res = BADGEI2C_SendBlock(&tmp, 6, &i2cret); // broadcast PWM values to all the other badges	
		  if (res != ERR_OK) // if transmission is not successful, restart I2C
		  {
				dc17_i2c_init();
		  }
		  
		  delay_ms(100);
  	  flags.ptr_update = 0;			
		}
	}
	*/
}


void dc17_i2c_slave(void)
{
	/* patrick : upon recognition as slave */
	
	/*Result 					res;
	word						i2cret; // return value from i2c function call	
  byte						tmp[6]; // temporary buffer for i2c tx/rx
	unsigned int		tmpw;

  badge_state = I2C;
	res = BADGEI2C_RecvBlock(&tmp, 6, &i2cret); // first data packet can be ignored
	flags.i2c_rx = 0;
	
	//Term1_SendStr("Multi-Badge Slave.\n\r");		
	
	// let's RECEIVE data...
	while(1) // ...forever!
	{
		while (!flags.i2c_rx){};

	  res = BADGEI2C_RecvBlock(&tmp, 6, &i2cret); // receive the bytes out of the buffer
	  if (res == ERR_OK)
	  {
			// convert into 16-bit values and set the PWM registers
		  tmpw = tmp[0];
		  tmpw <<= 8;
		  tmpw |= tmp[1];
		 	PWMRed_SetRatio16((unsigned int)tmpw);

		  tmpw = tmp[2];
		  tmpw <<= 8;
		  tmpw |= tmp[3];
			PWMGreen_SetRatio16((unsigned int)tmpw);

		  tmpw = tmp[4];
		  tmpw <<= 8;
		  tmpw |= tmp[5];
		 	PWMBlue_SetRatio16((unsigned int)tmpw);
	 	}
	  
	  flags.i2c_rx = 0;	
	}*/
}


void dc17_i2c_init(void) // initialize the I2C system for multi-badge communication
{
  //unsigned char i;
	
	//BADGEI2C_Disable(); // disable I2C module during configuration
	
	// initialize global flags
	flags.i2c_rx = 0;
	flags.i2c_nack = 0;
	
  // determine what type of badge I am (based on A2-A0 input values)
  /*i = 0;
  if (A2_GetVal()) i |= 1;
  i <<= 1;
  if (A1_GetVal()) i |= 1;
  i <<= 1;
  if (A0_GetVal()) i |= 1;
  i++;
  
  badge_type = (badge_type_struct)i;
  setReg(I2C_ADDR, (i << 1) & I2C_ADDR_AD_MASK); // set I2C slave address (upper 7 bits of register), must be non-zero  

	if (badge_type == HUMAN)
	{
	 	BADGEI2C_SetMode(TRUE); // human is always the master...
	 	
		// Set General Call Address - all subsequent calls of SendChar, SendBlock will be broadcast on the bus
		BADGEI2C_SelectSpecialCommand(0);
	}
  else
  {
   	BADGEI2C_SetMode(FALSE); // ...otherwise, we are a slave
  }*/
 
 /*
  Term1_SendStr("I am ");
  switch (badge_type)
  {
  	case UBER:
  		Term1_SendStr("Uber");
  		break;
  	case GOON:
  		Term1_SendStr("Goon");	
  		break;
  	case CONTEST:
  		Term1_SendStr("Contest");  	
  		break;
  	case SPEAKER:
  		Term1_SendStr("Speaker");  	
  		break;
  	case VENDOR:
  		Term1_SendStr("Vendor");  	
  		break;
  	case PRESS:
  		Term1_SendStr("Press");  	
  		break;
  	case HUMAN:
  		Term1_SendStr("Human");  	
  		break;
  	default:
  	  Term1_SendStr("Unused");
  	  break;
  }
	Term1_SendStr(".\n\r");
*/
	//BADGEI2C_Enable(); // enable I2C module
}


/**************************************************************/

void dc17_init(void) // badge start-up/initialization
{
  unsigned int 		options = FFT_SCALE_RESULTS_BY_N; // unconditionally scale down the results by n

	Cpu_DisableInt(); // disable global interrupts 	
	//PWMRed_Enable();  // while we enable all the modules
	//PWMGreen_Enable();
	//PWMBlue_Enable();
	MICOUT_Enable();

	Term1_SendStr("Welcome to the DEFCON 17 Blimp.\n\n\r");

  // take a listen at the current ambient noise
	/*MICOUT_Measure(TRUE); // number of conversions specified in ADC bean, wait for conversion to complete
	MICOUT_GetValue16(&micval); // get a/d sample  
	srand((unsigned int)micval); // ...seed the PRNG with the value */
  		
	TI0_Enable();
	//TI1_Enable();
	Cpu_EnableInt();  // re-enable global interrupts when we're done and ready for action
 		
	// initialize global flags
	flags.PerformFFT = FALSE;
	flags.ptr_update = 0;
	flags.sleep = 0;
	  
  threshold_failure = 0; // counters for auto-power-off
  threshold_count = 0;
	
	
	Term1_SendStr("Calibrating microphone, please be silent...");
	// Do real calibration here instead of this nonsense.
	/*locut[0] = 1000;
	locut[1] = 1000;
	locut[2] = 1000;*/
	
  // initialize the FFT algorithm
  dfr16RFFTInit (pRFFT, 8, options); // MAX_RFFT_LEN

	// initialize the I2C system for multi-badge communication
	//dc17_i2c_init();
	
}



/**************************************************************
/* INTERRUPT SERVICE ROUTINES
/**************************************************************/

void dc17_pit_isr(void) // Programmable Interval Timer (PIT): every 8ms, set to operate during Stop Mode
{  	   	
  //if (badge_state == FFT)
  //{
			// update the randomizer...  
			/*randtime++;
			if (randtime >= RAND_TMR) 
			{
				randval = rand() % 3; 
				randtime = 0;
			}*/
			
			// if noise energy is below our threshold for some amount of time, then go to sleep
 	/*		if (threshold_failure == 0)
 			{
 				threshold_count++;
 			}
 			else
 			{
 				threshold_count = 0;
 				threshold_failure = 0;
 			}
 		
 			if (threshold_count > SLEEP_TMR)
 				flags.sleep = 0;
 				//flags.sleep = 1;
 				
 			// increment counter for dance mode
 			soscount++; 
 	}
 	
  // LEDptr used for RGB pulsing and blending tables
	LEDptr++; // increment the pointer
	flags.ptr_update = 1;

	if (LEDptr >= LED_TabLength)  // if we've reached the end of the table, start again
		LEDptr = 0;*/
}


void dc17_t0_isr(void) // TMR0: A/D sampling, 8kHz (every 0.125ms) (Fmax = highest frequency that can be analyzed = Fs / 2 = 4kHz)
{
  if (badge_state == FFT)
  {  
		// get a/d sample and add it to our circular buffer
		MICOUT_Measure(TRUE); // number of conversions specified in ADC bean, wait for conversion to complete
		MICOUT_GetValue16(&micval);  	   
	  AdcBuffer[AdcWriteIndex] = micval;
	  AdcWriteIndex++;
	  
	  if (AdcWriteIndex >= ADC_MAX_BUFFER_SIZE) 
	  	AdcWriteIndex = 0; // wrap around to beginning of buffer
	  		
 		// check to see if it's time to do an FFT (e.g., we have enough new A/D samples)
    if (AdcReadIndex > AdcWriteIndex)
  	{
  		flags.PerformFFT = true;
  	}
  	else if (AdcReadIndex <= (int)(AdcWriteIndex - MAX_RFFT_LEN))
  	{
    	flags.PerformFFT = true;
  	}
  }
}


void dc17_t1_isr(void) // TMR1: 4Hz LED/Motor Timer
{

	if (isMaster)
	{
		
		if(master_led_ctr <= 3)
		{
			do_led_anim();
			//slave 1 off
			//slave 2 off
		}
		else if (master_led_ctr <= 6)
		{
			/*slave 1 high*/
			//slave 2 off
		}
		else if (master_led_ctr <= 9)
		{
			//slave 1 off
			//slave_2_hign
		}
		
		if (master_led_ctr >= 9)
		{
			master_led_ctr = 0;
		}
		master_led_ctr++;
	}
	else if (/*pin is high*/true)
	{
		do_led_anim();
	}
		

	motor_delay_ctr++;
	
	if (motor_delay_ctr <= 10)
	{
		motor_on = true;
		okay_to_listen=false;		
	}
	else if (motor_delay_ctr <= 14)
	{
		motor_on = false;
		okay_to_listen=false;
	}
	else if (motor_delay_ctr <= 16)
	{
		motor_on=false;
		okay_to_listen=true;
	}
	else
	{
		motor_delay_ctr=0;
	}
}

/**************************************************************
*	Function: 	delay
*	Parameters: 16-bit delay value
*	Return:		none
*
*	Simple delay loops
* These are not very accurate because they could get interrupted
**************************************************************/
 
void delay_ms(unsigned int ms)  // delay the specified number of milliseconds
{
  unsigned int i;

  for (i = 0; i < ms; ++i)
  	Cpu_Delay100US(10);
}

/****************************** END OF FILE **********************************/

